/* bgnd_fnc.c */

#include <dos.h>
#include <errno.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>

#include <at200.h>
#include <camera.h>
#include "idecls.h"
#include "camsys.h"
#include "proto.h"
#include "mode.h"

/* Values for dealing with watchdog timer */
#define watchdog_port	0x181
#define clr		1
#define off		0

extern	int loginterval;
int 	samplecount;

struct	diskfree_t df;

/* Carry out background temperature logging task at specified intervals. */

void chron_func( void )
{
	unsigned long	second;
static	unsigned long	lastsecond = 0L, 
			nextsamplesecond = 0L, 
			nextlogsecond = 0L;
	int	count, lastmin, kb_per_cluster;
	static  int	diskspace, diskfree;
	struct  dosdate_t sav_dt;
	struct  dostime_t sav_tm;

	_dos_getdate( &dt );
	if ( dt.day != ki_data.dt.day )		/* Pumpkin time! */
	{
	     memset( &ki_data, '\0', sizeof ( KI_DATA ) ); /* reset file seq */
             ki_data.dt = dt;				   /* Set new date */
        }
        lastmin = tm.minute;			/* Too see if minute changed */

	_dos_gettime( &tm );

	/* Get current time of day in seconds. */

	second = ((long) tm.hour * 3600L) + ((long) tm.minute * 60L) 
			+ (long) tm.second;

	/* Every time minute changes, calculate current diskspace. */

	if ( tm.minute != (unsigned char) lastmin )
	{	
		/* Calculate total and available diskspace in MB on C: */	

           _dos_getdiskfree ( 3, &df );
	   kb_per_cluster = df.sectors_per_cluster * df.bytes_per_sector/1024;
	   diskspace = (int)((long)df.total_clusters*(long)kb_per_cluster/1024L );
	   diskfree = (int)((long)df.avail_clusters *(long)kb_per_cluster/1024L );
	}

	/*  ***** CHECK HERE FOR DISK SPACE < THRESHOLD FOR TAPE BACKUP *** */

	if ( second < lastsecond )  /* Check for time of day clock wraparound. */
	{
		lastsecond = second;
		nextlogsecond = 0L;
		nextsamplesecond = 0L;
		return;
	}
	lastsecond = second;
	
	if ( !templogmode )		/* Return here if not doing */
		return;			/* temperature logging. */
		
	/* See if temperature sample interval time has expired. */

	if ( second > nextsamplesecond )	/* Time to take temp. sample. */
	{
		nextsamplesecond = second + (long) sampleinterval;
		sample_temperatures();
	}

	/* See if write temperature log interval time has expired. */

	if ( second > nextlogsecond )
	{
		/* Time to record temperature log data. */
		
		nextlogsecond = second + (long) loginterval;

		sprintf( omsg, "%02d:%02d:%02d.%02d   %4d/%-4d MB available", 
			tm.hour, tm.minute, tm.second, tm.hsecond,
			diskfree, diskspace );
		if ( !textmode )
			print_string( 322, 467, omsg, (char)140 );
		else
			printf("%s\n", omsg);
		if ( cmndlogmode ){
			fprintf( cmndlogfl, "%s\n", omsg );
			fflush( cmndlogfl );
		}
		
		if ( std0_misc.cam_opmode & 0x01 )
		{
		  sav_tm = std0_misc.tm;	/* Save exposure date/time */
		  sav_dt = std0_misc.dt;

		  std0_misc.tm = tm;	/* Plug in current time. */
		  std0_misc.dt = dt;
			
		  /* Write the temperature data for both cameras. */
		
		  count = write( templogfl, &std0_misc, sizeof ( std0_misc ) );
		  if ( count != sizeof ( std0_misc ))
		  {
		     sprintf( omsg, "ERROR: Unable to write temperature log\n"); 
		     disp_omsg( ERROR );
		  }

		std0_misc.tm = sav_tm;	/* Restore exposure date/time */
		std0_misc.dt = sav_dt;
	        }
		
	        if ( std1_misc.cam_opmode & 0x02 )
	        {
		  sav_tm = std1_misc.tm;	/* Save exposure date/time */
   		  sav_dt = std1_misc.dt;

		  std1_misc.tm = tm;	/* Plug in current time. */
		  std1_misc.dt = dt;
		
		  count = write( templogfl, &std1_misc, sizeof ( std1_misc ) );
		  if ( count != sizeof ( std1_misc ))
		  {
		     sprintf( omsg, "ERROR: Unable to write temperature log\n"); 
		     disp_omsg( ERROR );
		  }

		  std1_misc.tm = sav_tm;	/* Restore exposure date/time */
		  std1_misc.dt = sav_dt;
	        }

                close( duptlogfl );		

		/* Reset temperature data */
		
		memset( &std0_misc.temp, '\0', sizeof( temp_t ) );
		memset( &std1_misc.temp, '\0', sizeof( temp_t ) );
	}
}



/* This procedure reads all temperatures for both camera units, 
 * saves them in the camera structures.
 */

void sample_temperatures( void )
{
	pv_ptr p = (void *) NULL; /* used for linkage to temperature functs */
	int save_camera;

	if ( std0_misc.cam_opmode & 0x1 )	/* If camera 0 enabled. */
	{
		save_camera = sel_cam;		/* Save current camera # */
		select_camera( 0 );		/* Read camera 0 temps. */
		inhibit_display = inhibit0;	/* Prevent temperature display */
		p = afb_id ( p );		/* Store AFB ID */
		p = tfw ( p );			/* Log temperature readings */
		p = tei ( p );
		p = td  ( p );
		p = tee ( p );
		p = tcf ( p );
		p = heater ( p );		/* Heater on/off status */
		p = fan_stat ( p );		/* Fan status */
		p = acpower ( p );
		p = plus5 ( p );
		p = plus15 ( p );
		p = minus15 ( p );
		  t_history0[sampleptr].tei = std0_misc.temp.tei[TEMP_CUR];
		  t_history0[sampleptr].tee = std0_misc.temp.tee[TEMP_CUR];
		  t_history0[sampleptr].td  = std0_misc.temp.td[TEMP_CUR];
		  t_history0[sampleptr].tcf = std0_misc.temp.tcf[TEMP_CUR];
		  t_history0[sampleptr].tfw = std0_misc.temp.tfw[TEMP_CUR];
		  t_history0[sampleptr].tm  = tm;
		select_camera( save_camera );	/* re-select current camera */
	}

	if ( std1_misc.cam_opmode & 0x2 )	/* If camera 1 enabled. */
	{
		save_camera = sel_cam;		/* Save current camera # */
		select_camera( 1 );		/* Read camera 1 temps. */
		inhibit_display = inhibit1;
		p = afb_id ( p );
		p = tfw ( p );
		p = tei ( p );
		p = td  ( p );
		p = tee ( p );
		p = tcf ( p );
		p = heater ( p );
		p = fan_stat ( p );
		p = acpower ( p );
		p = plus5 ( p );
		p = plus15 ( p );
		p = minus15 ( p );
		  t_history1[sampleptr].tei = std1_misc.temp.tei[TEMP_CUR];
		  t_history1[sampleptr].tee = std1_misc.temp.tee[TEMP_CUR];
		  t_history1[sampleptr].td  = std1_misc.temp.td[TEMP_CUR];
		  t_history1[sampleptr].tcf = std1_misc.temp.tcf[TEMP_CUR];
		  t_history1[sampleptr].tfw = std1_misc.temp.tfw[TEMP_CUR];
		  t_history1[sampleptr].tm  = tm;
		select_camera( save_camera );	/* re-select current camera */
	}
		sampleptr++; no_samples++;
		if ( sampleptr > MAX_SAMPLES )
		  sampleptr = 0;
		if ( no_samples > MAX_SAMPLES )
		  no_samples = MAX_SAMPLES;
	inhibit_display = 0;		/* re-enable console display */
}
